package OTRS::OPM::Maker::Command::dbtest;

# ABSTRACT: Test db definitions in .sopm files

use strict;
use warnings;

use OTRS::OPM::Maker -command;

our $VERSION = 0.07;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

OTRS::OPM::Maker::Command::dbtest - Test db definitions in .sopm files

=head1 VERSION

version 0.1

=head1 AUTHOR

Renee Baecker <reneeb@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2016 by Renee Baecker.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut
